//----------------------------------------------------------------------
// vx-scheme : Scheme interpreter.
// Copyright (c) 2002,2003 and onwards Colin Smith.
//
// You may distribute under the terms of the Artistic License, 
// as specified in the LICENSE file.
//
// u-main.cpp : startup code for UNIX, Cygwin or Win32 environments.

#include <stdio.h>
#include <time.h>
#include "vx-scheme.h"
#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#else
#include <windows.h>
#endif
#include <setjmp.h>

static jmp_buf jb;


//----------------------------------------------------------------------------
//
// OS-SPECIFIC FEATURES
//
// This area fills in definitions for OS-specific features named 
// in class OS.  
//

void OS::get_time (int & sec, int & usec)
    {
#ifdef WIN32
	FILETIME filetime;
	GetSystemTimeAsFileTime(&filetime);
	ULARGE_INTEGER ul;
	ul.HighPart = filetime.dwHighDateTime;
	ul.LowPart = filetime.dwLowDateTime; 
	// FILETIMES  are in 100ns units.
	sec = static_cast<int>(ul.QuadPart / 100000000);
	int hnsec = static_cast<int>(ul.QuadPart % 100000000);
	usec = hnsec / 10;
#else
	struct timeval t;
    gettimeofday (&t, 0);
    sec = t.tv_sec;
    usec = t.tv_usec;
#endif
    }

unsigned int OS::flags ()
    {
    static bool env_checked = false;
    static unsigned int f = 0;
    if (! env_checked)
	{
	char * c;
	if ((c = getenv ("T")) != NULL)
	    f = atoi (c);
	env_checked = true;
	}

    return f;
    }

bool OS::interactive (int fd)
    {
    return isatty (fd) != 0;
    }

Cell * OS::undef (Context * ctx, const char * name)
    {
    return 0;
    }

void OS::exception (const char * s)
    {
    longjmp (jb, reinterpret_cast <int> (s));
    }

void interact (Context * ctx)
    {
    bool interactive = OS::interactive(0);

    while (ctx->read_eval_print (stdin, stdout, interactive))
	;
    
    if (OS::flag (DEBUG_MEMSTATS_AT_EXIT)) { 
	ctx->print_mem_stats (stdout);
        Cell::stats ();
    }


    exit (0);
    }

int main
    (
    int         argc,
    char *      argv []
    )
    {
    const char *    jv;
    Context         ctx;

    // establish error-handling context
    if ((jv = reinterpret_cast <const char *> (setjmp (jb))) == 0)
	{ 
	// If arguments were specified, treat them as expressions to 
	// evaluate.  Otherwise REPL from standard input.
	if (argc > 1)
	    {
	    for (int ix = 1; ix < argc; ++ix)
		{
		sstring ss;
		ss.append (argv [ix]);
		while (! ss.eof ())
		    {
		    Cell * result = ctx.eval (ctx.read (ss));
		    if (result != unspecified)
			{ 
			result->write (stdout);
			fputc ('\n', stdout);
			}
		    }
		}
	    }
	else
	    while (1)
		interact (&ctx);
	}
    else
	fprintf (stderr, "caught: %s\n", jv);
    }


